clear; %clear workspace

fontsize = 16;

%parameters to test
%delta_f=1;
%delta_f=7;
delta_f=0;
phase_step=pi;
step_time=0;
initial_phase=0;

%initial condition
f_ref=10;
f0_vco=f_ref-delta_f;

%PLL design constants:
zeta=0.707;
omega_n=2*pi;

kd=1; %phase detector gain
k0=2*pi; %vco gain
Ni=1;
No=1;

%simulation settings
sim_end_time=2;
points_per_second=1000;
N=sim_end_time*points_per_second; %number of samplepoints


%constant calculation for controller
k1=(2*zeta*omega_n*No)/(k0*kd);
k2=(omega_n^2*No)/(k0*kd);
k=k0*kd*k1;
disp(strcat('k=',num2str(k)));

if zeta < 1
    omega_r=omega_n*sqrt(1-zeta^2);
    phi=atan(sqrt(1-zeta^2)/zeta);
    max_delta_f=omega_r*Ni*exp(zeta*omega_n/omega_r*phi)/(2*abs(sin(phi)));
    disp(strcat('max. delta_f=',num2str(max_delta_f)));
    disp(strcat('approx max. delta_f=',num2str(omega_n*Ni*0.8594*zeta+0.5)));
end

%phase model:
time = 0:sim_end_time/(N-1):sim_end_time;
if delta_f > 0
    u_end=delta_f*2*pi*sim_end_time;
    u = 0:u_end/(N-1):u_end;
    error_tf2 = tf([0 0 (1/Ni)*delta_f*2*pi],[1 2*zeta*omega_n omega_n^2]); %1. arg z�hler,2. arg nenner
    [phase_error_th,time] = impulse(error_tf2,time);
else
    error_tf = tf([(1/Ni) 0 0],[1 2*zeta*omega_n omega_n^2]); %1. arg z�hler,2. arg nenner
    [phase_error_th,time] = step(error_tf,time);
    phase_error_th = phase_error_th * phase_step;
end


%simulate:
sim_opts = simset('FixedStep',sim_end_time/(N-1));

sim('continuous_ideal_pll1',sim_end_time,sim_opts); %[t,x,y] = sim(model, timespan, options, ut);

if delta_f > 0
    title_phase_error='Frequenzsprungantwort';
else
    title_phase_error='Phasensprungantwort';
end

fig(1)=figure(1)
clf reset
axes1 = axes('Parent',fig(1),'FontSize',fontsize);
plot(sim_time,-phase_error*180/pi,'--k',sim_time,phase_error_th*180/pi,':k')
%title(strcat(title_phase_error,' - Phasenfehler'));
xlabel('t [s]')
ylabel('\Delta\phi(t) []')
legend({'Amplitudenmodell','Phasenmodell'});
grid

fig(2)=figure(2)
clf reset
axes2 = axes('Parent',fig(2),'FontSize',fontsize);
plot(sim_time,abs(freq_error),'--k',sim_time,ones(N,1)*delta_f,'-.k')
xlabel('t [s]')
ylabel('\Delta f(t) [Hz]')
%title(strcat(title_phase_error,' - Ausgangsfrequenz'));
legend({'\Delta f_{o}','\Delta f_{i}'});
grid

disp(strcat('max. error between simulation and analythical res.=',num2str(max(phase_error-phase_error_th))));

if delta_f == 0
    hgexport(fig(1),'../../../tex/images/continuous_ideal_pll_phase_step_phase_error.eps')
else
    if delta_f == 1
        hgexport(fig(1),'../../../tex/images/continuous_ideal_pll_freq_step_phase_error.eps')
        hgexport(fig(2),'../../../tex/images/continuous_ideal_pll_freq_step_output_freq.eps')
    else
        if delta_f == 7
            hgexport(fig(1),'../../../tex/images/continuous_ideal_pll_freq_step_7Hz_phase_error.eps')
            hgexport(fig(2),'../../../tex/images/continuous_ideal_pll_freq_step_7Hz_output_freq.eps')
        end;
    end;
end;    



